/* 
 * File: TSPUtil.java
 *
 * Created on 23/10/2009
 *
 *
 * Copyright 2006-2007 Felix Garcia Borrego (borrego@gmail.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.viafirma.util;

import java.security.Key;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Security;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.jce.provider.BouncyCastleProvider;


public class RSAUtil {

    private static RSAUtil rsaUtil = null;

    public static RSAUtil getInstance() {
	if (rsaUtil == null) {
	    Security.addProvider(new BouncyCastleProvider());
	    rsaUtil = new RSAUtil();
	}
	return rsaUtil;
    }

    private byte[] append(byte[] prefix, byte[] suffix) {
	byte[] toReturn = new byte[prefix.length + suffix.length];
	for (int i = 0; i < prefix.length; i++) {
	    toReturn[i] = prefix[i];
	}
	for (int i = 0; i < suffix.length; i++) {
	    toReturn[i + prefix.length] = suffix[i];
	}
	return toReturn;
    }

    public String encrypt(String plaintext, RSAPublicKey iccPubkey) throws Exception {

	Cipher cipher = Cipher.getInstance("RSA/None/NoPadding", "BC");
	cipher.init(Cipher.ENCRYPT_MODE, iccPubkey);
	byte[] bytes = plaintext.getBytes(Constantes.ENCODING);

	byte[] encrypted = blockCipher(bytes, Cipher.ENCRYPT_MODE, cipher);

	char[] encryptedTranspherable = Hex.encodeHex(encrypted);
	return new String(encryptedTranspherable);
    }

    public String decrypt(String encrypted, RSAPrivateKey iccPrivkey) throws Exception {

	Cipher cipher = Cipher.getInstance("RSA/None/NoPadding", "BC");
	cipher.init(Cipher.DECRYPT_MODE, iccPrivkey);
	byte[] bts = Hex.decodeHex(encrypted.toCharArray());

	byte[] decrypted = blockCipher(bts, Cipher.DECRYPT_MODE, cipher);

	return new String(decrypted, Constantes.ENCODING);
    }

    private byte[] blockCipher(byte[] bytes, int mode, Cipher cipher) throws IllegalBlockSizeException, BadPaddingException {

	// string initialize 2 buffers.
	// scrambled will hold intermediate results
	byte[] scrambled = new byte[0];

	// toReturn will hold the total result
	byte[] toReturn = new byte[0];
	// if we encrypt we use 100 byte long blocks. Decryption requires 128 byte long blocks (because of RSA)
	int length = (mode == Cipher.ENCRYPT_MODE) ? 100 : 256;

	// another buffer. this one will hold the bytes that have to be modified in this step
	byte[] buffer = new byte[length];

	for (int i = 0; i < bytes.length; i++) {

	    // if we filled our buffer array we have our block ready for de- or encryption
	    if ((i > 0) && (i % length == 0)) {
		// execute the operation
		scrambled = cipher.doFinal(buffer);
		// add the result to our total result.
		toReturn = append(toReturn, scrambled);
		// here we calculate the length of the next buffer required
		int newlength = length;

		// if newlength would be longer than remaining bytes in the bytes array we shorten it.
		if (i + length > bytes.length) {
		    newlength = bytes.length - i;
		}
		// clean the buffer array
		buffer = new byte[newlength];
	    }
	    // copy byte into our buffer.
	    buffer[i % length] = bytes[i];
	}

	// this step is needed if we had a trailing buffer. should only happen when encrypting.
	// example: we encrypt 110 bytes. 100 bytes per run means we "forgot" the last 10 bytes. they are in the buffer array
	scrambled = cipher.doFinal(buffer);

	// final step before we can return the modified data.
	toReturn = append(toReturn, scrambled);

	return toReturn;
    }

    /**
     * Recupera el KeyPair que contiene la clave privada y la clave pública de un certificado
     * 
     * @param keystore
     * @param alias
     * @param password
     * @return
     */
    public KeyPair getKeyPair(KeyStore keystore, String alias, char[] password) {
	try {
	    Key key = keystore.getKey(alias, password);
	    if (key instanceof PrivateKey) {
		Certificate cert = keystore.getCertificate(alias);
		if (cert != null) {
		    PublicKey publicKey = cert.getPublicKey();
		    return new KeyPair(publicKey, (PrivateKey) key);
		} else {
		    throw new UnrecoverableKeyException("Unable to recover certificate. Keystore return null for alias: " + alias);
		}
	    }
	} catch (UnrecoverableKeyException e) {
	    log.error(e.getMessage(), e);
	} catch (NoSuchAlgorithmException e) {
	    log.error(e.getMessage(), e);
	} catch (KeyStoreException e) {
	    log.error(e.getMessage(), e);
	}
	return null;
    }

    private Log log = LogFactory.getLog(RSAUtil.class);

}
