package org.viafirma.test.encrypt;

import java.io.File;
import java.io.IOException;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.interfaces.RSAPrivateKey;
import java.util.Enumeration;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.xml.security.utils.Base64;
import org.viafirma.util.RSAUtil;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class TestBiometricDecryption {

    /**
     * Get key pair from .p12 file
     * @param certFilePath
     * @param certPassword
     * @return KeyPair - Private and public key from certificate
     * @throws KeyStoreException
     * @throws NoSuchAlgorithmException
     * @throws CertificateException
     * @throws IOException
     */
    private static KeyPair getKeyPair(String certFilePath, String certPassword) throws KeyStoreException, NoSuchAlgorithmException, CertificateException, IOException {
        KeyStore ks = KeyStore.getInstance("pkcs12");
        ks.load(TestBiometricDecryption.class.getResourceAsStream(certFilePath), certPassword.toCharArray());
        Enumeration<String> aliases = ks.aliases();
        KeyPair keyPair = null;
        String certAlias = null;
        while (aliases.hasMoreElements()) {
            certAlias = aliases.nextElement();
            keyPair = RSAUtil.getInstance().getKeyPair(ks, certAlias, certPassword.toCharArray());
        }

        return keyPair;
    }

    /**
     * Decrypt biometric data with RSA private key
     * @param biometricData - Encrypted biometric data
     * @return String - Decrypted data
     * @throws Exception
     */
    private static String decrypt(String biometricData, KeyPair keyPair) throws Exception {
        byte[] biometricDataBA = Base64.decode(biometricData);
        String encryptedData = new String(biometricDataBA);
        String decrypted = RSAUtil.getInstance().decrypt(encryptedData, (RSAPrivateKey) keyPair.getPrivate());

        return decrypted;
    }

    /**
     * Read XML file
     * @param xmlFile
     * @return Document
     * @throws SAXException
     * @throws IOException
     * @throws ParserConfigurationException
     */
    private static Document readXml(File xmlFile) throws SAXException, IOException, ParserConfigurationException {
        DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = domFactory.newDocumentBuilder();
        Document doc = builder.parse(xmlFile.getAbsolutePath());
        doc.getDocumentElement().normalize();

        return doc;
    }

    /**
     * Extract biometric data (data node) from XML file
     * @param xmlDoc
     * @return String - Biometric data in Base64/RSA
     * @throws XPathExpressionException
     */
    private static String extractBiometricData(Document xmlDoc) throws XPathExpressionException {
        XPathFactory xPathfactory = XPathFactory.newInstance();
        XPath xpath = xPathfactory.newXPath();

        XPathExpression expr = xpath.compile("//biometric/data/text()");
        String biometricData = (String) expr.evaluate(xmlDoc, XPathConstants.STRING);

        return biometricData;
    }

    
    public static void main(String[] args) {
        String xmlFilePath = "/home/ruben/wacom_biometricData.xml";
        // String xmlFilePath = "/home/ruben/android_biometricData.xml";
        //String xmlFilePath = "/home/ruben/ios_biometricData.xml";

        try {
            Document xmlDoc = readXml(new File(xmlFilePath));

            // Extract Base64/RSA biometric data from XML
            String biometricData = extractBiometricData(xmlDoc);

            // Get key pair from p12 file
            KeyPair keyPair = getKeyPair("/certificados/xnoccio.p12", "12345");

            // Decrypt biometric data with private key
            String decrypted = decrypt(biometricData, keyPair);
            System.out.println("Decrypted: \n" + decrypted);

            // Only for test purpose. Encrypt again with public key for
            // comparison. It must be equal to original biometric data base64
            // String encrypted = RSAUtil.getInstance().encrypt(decrypted,
            // (RSAPublicKey) keyPair.getPublic());
            // encrypted = Base64.encode(encrypted.getBytes());
            // System.out.println("Encrypted \n" + encrypted);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

}
